# -*- coding: iso-8859-1 -*-

TXEnableDelay = 2e-6
TXEnableValue = 0b0001  # TTL line blanking RF amplifier (bit 0)
TXPulseValue  = 0b0010  # TTL line triggering RF pulses  (bit 1)
ADCSensitivity = 2	# voltage span for ADC

def experiment(): # Jeener-Broekaert echo sequence (a.k.a. spin-alignment)

  # set up acquisition parameters:         
    pars = {}
    pars['P90'] = 2.3e-6	# 90-degree pulse length (s)
    pars['SF'] = 46.7e6		# spectrometer frequency (Hz)
    pars['O1'] = -30e3	 	# offset from SF (Hz)
    pars['SW'] = 1e6    	# spectral window (Hz)
    pars['SI'] = 1*512  	# number of acquisition points
    pars['NS'] = 8		# number of scans
    pars['DS'] = 0		# number of dummy scans
    pars['RD'] = 3		# delay between scans (s)
    pars['D1'] = 30e-6		# delay after first pulse, or tp (s)
    pars['D2'] = 100e-6		# delay after second pulse, or tm (s)
    pars['PHA'] = -36		# receiver phase (degree)
    pars['DATADIR'] = '/home/fprak/Students/'	# data directory
    pars['OUTFILE'] = None			# output file name
    
  # specify a variable parameter (optional):
    pars['VAR_PAR'] = 'D2'	# variable parameter name (a string)   
    start = 30e-6		# starting value
    stop = 2e-0			# end value
    steps = 24			# number of values
    log_scale = True		# log scale flag 
    stag_range = False		# staggered range flag

  # check parameters for safety:
    if pars['PHA'] < 0:
        pars['PHA'] = 360 + pars['PHA']
        
    if pars['P90'] > 20e-6:
        raise Exception("Pulse too long!!!")
                    
    # check whether a variable parameter is named: 
    var_key = pars.get('VAR_PAR')
    if var_key == 'P90' and (start > 20e-6 or stop > 20e-6):
        raise Exception("Pulse too long!!!")

    if pars['NS']%8 != 0:
        pars['NS'] = int(round(pars['NS'] / 8) + 1) * 8 
        print 'Number of scans changed to ', pars['NS'], ' due to phase cycling'

    # start the experiment:
    if var_key: 
        # this is an arrayed experiment:
        if log_scale:
            array = log_range(start,stop,steps)
        else:
            array = lin_range(start,stop,steps) 
        
        if stag_range:
            array = staggered_range(array, size = 2)

       # estimate the experiment time:
        if var_key == 'D1':
            seconds = (sum(array)*2 + (pars['D2'] + pars['RD']) * steps) * (pars['NS'] + pars['DS'])
        elif var_key == 'D2':
            seconds = (sum(array) + (pars['D1']*2 + pars['RD']) * steps) * (pars['NS'] + pars['DS'])
        elif var_key == 'RD':
            seconds = (sum(array) + (pars['D1']*2 + pars['D2']) * steps) * (pars['NS'] + pars['DS'])
        else:
            seconds = (pars['D1']*2 + pars['D2'] + pars['RD']) * steps * (pars['NS']+ pars['DS'])
        m, s = divmod(seconds, 60)
        h, m = divmod(m, 60)
        print '%s%02d:%02d:%02d' % ('Experiment time estimated: ', h, m, s)

        # loop for a variable parameter:
        for index, pars[var_key] in enumerate(array):
            print 'Arrayed experiment for '+var_key+': run = '+str(index+1)+\
                             ' out of '+str(array.size)+': value = '+str(pars[var_key])
            # loop for accumulation:
            for run in xrange(pars['NS']+pars['DS']):
                yield spinal_experiment(pars, run)
            synchronize()

    else:
       # estimate the experiment time:
        seconds = (pars['D1']*2 + pars['D2'] + pars['RD']) * (pars['NS']+ pars['DS'])
        m, s = divmod(seconds, 60)
        h, m = divmod(m, 60)
        print '%s%02d:%02d:%02d' % ('Experiment time estimated: ', h, m, s)

       # loop for accumulation:
        for run in xrange(pars['NS']+pars['DS']):
            yield spinal_experiment(pars, run)


# the pulse program: 

def spinal_experiment(pars, run):
    e=Experiment()

    dummy_scans = pars.get('DS')
    if dummy_scans:
        run -= dummy_scans
                       
    pars['PROG'] = 'spinal_experiment'                       
                       
   # 8-step phase cycle (1-14 modifided to deal with T1-recovery and extended for Re/Im imbalance)
    pars['PH1'] = [0, 270,   0, 270,     90,  90, 180, 180 ] # 1st (90-degree) pulse
    pars['PH3'] = [90,180,  90, 180,    180, 180,  90,  90 ] # 2nd (45-degree) pulse
    pars['PH4'] = [90, 90, 270, 270,    180,   0,   0, 180 ] # 3rd (45-degree) pulse
    pars['PH2'] = [0, 180, 180,   0,     90, 270,  90, 270 ] # receiver
        
   # read in variables:
    P90 = pars['P90']
    P45 = pars['P90']*0.5
    P1  = pars['P90']*0.5
    SF =  pars['SF']
    O1 =  pars['O1']    
    RD =  pars['RD']
    D1 =  pars['D1']
    D2 =  pars['D2']
    PH1 = pars['PH1'][run%len(pars['PH1'])]
    PH3 = pars['PH3'][run%len(pars['PH3'])]
    PH4 = pars['PH4'][run%len(pars['PH4'])]
    PH2 = pars['PH2'][run%len(pars['PH2'])]
    PHA = pars['PHA']

   # set sampling parameters:
    SI = pars['SI']
    SW = pars['SW']
    while SW <= 10e6 and SI < 256*1024:
        SI *= 2
        SW *= 2

   # run the pulse sequence:
    e.wait(RD)						# relaxation delay between scans
    e.set_frequency(SF+O1, phase=PH1)
    e.ttl_pulse(TXEnableDelay, value=TXEnableValue)
    e.ttl_pulse(P90, value=TXEnableValue|TXPulseValue)	# 90-degree pulse 

    e.wait(D1-P90/2-TXEnableDelay)			# 'short tau'
    e.set_phase(PH3)

    e.ttl_pulse(TXEnableDelay, value=TXEnableValue)
    e.ttl_pulse(P45, value=TXEnableValue|TXPulseValue)	# 45-degree pulse 

    e.wait(D2-P45/2-TXEnableDelay)			# 'long tau'
    e.set_phase(PH4)

    e.ttl_pulse(TXEnableDelay, value=TXEnableValue)
    e.ttl_pulse(P1, value=TXEnableValue|TXPulseValue)	# 45-degree pulse

    e.wait(TXEnableDelay)
    e.set_phase(PHA)
    e.wait(5e-6)#D1-P45/2-TXEnableDelay)    	                # 'short tau'
    e.record(SI, SW, sensitivity=ADCSensitivity)        # acquisition                                     

   # write experiment parameters:
    for key in pars.keys():
        e.set_description(key, pars[key])	# acquisition parameters
    e.set_description('run', run)		# current scan
    e.set_description('rec_phase', -PH2)	# current receiver phase

    return e